/**
 * This file is part of cs838-dc1

 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs838-dc1 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#include "bar_diff.h"
#include "frame.h"

#include <QColor>
#include <QBrush>
#include <QPen>
#include <QRect>

#include <cfloat>
#include <cmath>
#include <iostream>
#include <cstdlib>
using namespace std;

BarDiff::BarDiff(QWidget * parent) : DiffWidget(parent)
{ 

}

BarDiff::~BarDiff()
{

}

DiffWidget * BarDiff::create(const vector<Frame *> & frames)
{
	if(frames.size() < 1)
		return 0;

	BarDiff * newDiff = new BarDiff;

	// we'll need a pointer for reference
	newDiff->ref = frames[0];

	newDiff->frameCount = frames.size();

	// fill in the vector of weights and determin the max height
	newDiff->maxHeight = FLT_MIN;
	for(int r = 0; r < frames[0]->size(); r++)
	{
		vector< vector<float> > rTemp;
		newDiff->heights.push_back(rTemp);	
		for(int c = 0; c < frames[0]->size(); c++)
		{
			vector<float> cTemp;
			newDiff->heights[r].push_back(cTemp);
			for(unsigned int d = 0; d < frames.size(); d++)
			{
				newDiff->heights[r][c].push_back((*frames[d])(r,c));
				if((*frames[d])(r,c) > newDiff->maxHeight)
					newDiff->maxHeight = (*frames[d])(r,c);
			}
		}
	}

	// fill in colors for each frame
	QColor color;
	for(unsigned int i = 0; i < frames.size(); i++)
	{
		color.setRgb(rand() % 256, rand() % 256, rand() % 256);
		newDiff->colors.push_back(color);
	}

	return newDiff;	
}
		
void BarDiff::draw()
{
	QBrush brush;
	QPen pen;


	// fill in the back ground with white
	brush.setStyle(Qt::SolidPattern);
	brush.setColor(QColor(255, 255, 255));
	painter.fillRect(0, 0, width(), height(), brush);

	// calc some usefull data	
	int wGridStep = width() / (heights.size() + 1);
	int hGridStep = height() / (heights.size() + 1);

	// calculate some usefull stuff
	int barStep = wGridStep / frameCount;
	float barScale = static_cast<float>(hGridStep) / maxHeight;

	// go through each node and draw the color
	for(unsigned int r = 0, h = hGridStep; r < heights.size(); r++, h += hGridStep)
	{
		for(unsigned int c = 0, w = wGridStep; c < heights.size(); c++, w += wGridStep)
		{
			int bw = c * wGridStep + wGridStep;
			for(unsigned int i = 0; i < heights[r][c].size(); i++, bw += barStep)
			{
				int barHeight = static_cast<int>(barScale * heights[r][c][i]);
				int bh = h + (hGridStep - barHeight);
				brush.setColor(colors[i]);
				painter.fillRect(bw, bh, barStep, barHeight, brush);
			}
		}
	}


	// fill in the grid lines
	pen.setColor(QColor(100, 100, 100));

	// draw in the vertical grid lines
	for(int i = 0, w = wGridStep, h = hGridStep; 
		i < static_cast<int>(heights.size()); i++, 
		w += wGridStep, h += hGridStep)
	{
		if(i == 0)
			pen.setWidth(5);
		else
			pen.setWidth(1);

		painter.setPen(pen);

		painter.drawLine(w, 0, w, height());
		painter.drawLine(0, h, width(), h);
	}
	

	// draw in the labels
	QRect boundBox;

	for(int i = 1; i <= ref->size(); i++)
	{
		// draw the row
		boundBox.setRect(i * wGridStep, 0, wGridStep, hGridStep);
		pen.setColor(QColor(100, 100, 100));
		painter.setPen(pen);
		painter.drawText(boundBox, Qt::AlignHCenter | Qt::AlignVCenter, 
					ref->getConcept(i - 1));

		// draw the column
		boundBox.setRect(0, i * hGridStep , wGridStep, hGridStep);
		painter.drawText(boundBox, Qt::AlignHCenter | Qt::AlignVCenter, 
					ref->getConcept(i - 1));
	}
	

}
