/**
 * This file is part of cs838-dc1

 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs838-dc1 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#include "diff_factory.h"

#include "frame.h"
#include "diff_widget.h"

#include <map>
#include <iostream>
using namespace std;

// include diff widgets here
#include "lightness_diff.h"
#include "bar_diff.h"

// a map that maps a QString to a Widget name
static map<QString, DiffWidget*> localDiffWidgets;
static bool diffsLoaded = false;

// add diff widgets to the map here
static void loadDiffs()
{
	localDiffWidgets["Lightness"] = new LightnessDiff;
	localDiffWidgets["Bar"] = new BarDiff;
}

void getDiffWidgets(const vector<Frame *> & frames, map<QString, DiffWidget*> & diffWidgets)
{
	if(!diffsLoaded)
	{
		loadDiffs();
		diffsLoaded = true;
	}

	// iterate through diffs and add any that successfully create
	for(map<QString, DiffWidget*>::iterator iter = localDiffWidgets.begin(); 
			iter != localDiffWidgets.end(); iter++)
	{
		DiffWidget * newWidget = iter->second->create(frames);
		if(newWidget)
			diffWidgets[iter->first] = newWidget;
	}			

	return;
}
