/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */

#include "diff_widget.h"
#include "frame.h"

#include <QPixmap>
#include <QResizeEvent>
#include <QPaintEvent>


DiffWidget::DiffWidget(QWidget * parent) : QWidget(parent), viewImage(0), redrawImage(true)
{

}

DiffWidget::~DiffWidget()
{
	if(viewImage)
		delete viewImage;
}
		
void DiffWidget::resizeViewImage()
{
	if(viewImage)
		delete viewImage;
	
	viewImage = new QPixmap(width(), height());
}

void DiffWidget::resizeEvent(QResizeEvent*)
{
	resizeViewImage();
	redrawImage = true;
	update();
}
	
void DiffWidget::paintEvent(QPaintEvent*)
{
	if(viewImage->width() != width() || viewImage->height() != height())
		resizeViewImage();

	// call our drawing function
	if(redrawImage)
	{
		painter.begin(viewImage);
		this->draw();
		painter.end();
		redrawImage = false;
	}
	
	// draw the image using a painter
	painter.begin(this);
	painter.drawPixmap(0, 0, viewImage->width(), viewImage->height(), *viewImage);
	painter.end();
}
