/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */


#ifndef DIFF_WIDGET_H
#define DIFF_WIDGET_H

#include <QWidget>
#include <QPainter>

class Frame;
class QPixmap;
class QResizeEvent;
class QPaintEvent;

#include <vector>
using namespace std;

/**
 * a pure virtual class that will serve as the base class 
 * for all diff widgets
 */
class DiffWidget : public QWidget
{
	public:
		DiffWidget(QWidget * parent = 0);
		virtual ~DiffWidget();

		virtual DiffWidget * create(const vector<Frame *> & frames) = 0;

	protected:
		// the sub classes will implement this and the 
		// paint event handler will call it after
		// making sure the image is the correct size
		virtual void draw() = 0;

		// this can be used by sub classes to draw the vis
		QPixmap * viewImage;

		// the painter will initialy paint to the viewImage
		// but can be used by the child class to paint to other surfaces
		QPainter painter;

		// a varriable that can be used to flag the drawing function
		// to redraw the back buffered image
		bool redrawImage;

	private:
		// we'll handel the resize event here so that we can resize the viewImage	
		// this will always call update
		virtual void resizeEvent(QResizeEvent * event);
		virtual void paintEvent(QPaintEvent * event);

		void resizeViewImage();
};

#endif 
