/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */


#include "lightness_diff.h"
#include "frame.h"

#include <QColor>
#include <QBrush>
#include <QPen>
#include <QRect>

#include <cfloat>
#include <cmath>
#include <iostream>
using namespace std;

LightnessDiff::LightnessDiff(QWidget * parent) : DiffWidget(parent), diff(0)
{ 

}

LightnessDiff::~LightnessDiff()
{
	if(diff)
		delete diff;
}

DiffWidget * LightnessDiff::create(const vector<Frame *> & frames)
{
	if(frames.size() < 2)
		return 0;

	LightnessDiff * newDiff = new LightnessDiff;
	
	newDiff->diff = new Frame;
	subtract(*frames[0], *frames[1], *(newDiff->diff));
	newDiff->title = frames[0]->name() + " - " + frames[1]->name();

	return newDiff;	
}
		
void LightnessDiff::draw()
{
	QBrush brush;
	QPen pen;

	// fill in the back ground with white
	brush.setStyle(Qt::SolidPattern);
	brush.setColor(QColor(255, 255, 255));
	painter.fillRect(0, 0, width(), height(), brush);

	// get a bounding box for a title
	QRect titleBound = painter.boundingRect(QRect(0, 0, 1000, 1000), 
				Qt::AlignVCenter | Qt::AlignHCenter, title);
	int hStart = titleBound.height() * 2;
	
	// draw the title
	pen.setColor(QColor(100, 100, 100));
	painter.setPen(pen);
	QRect tBound(0, 0, width(), titleBound.height() * 2);
	painter.drawText(tBound, Qt::AlignVCenter | Qt::AlignHCenter, title);

	// calc some usefull data	
	int wGridStep = width() / (diff->size() + 1);
	int hGridStep = (height() - hStart) / (diff->size() + 1);

	// fill in the colors
	// determin the value with the largest magnitude
	float max = FLT_MIN;
	for(int r = 0; r < diff->size(); r++)
	{
		for(int c = 0; c < diff->size(); c++)
		{
			//cout << "max: " << max << " - diff " << fabs((*diff)(r,c)) << endl;
			max = (fabs((*diff)(r,c)) > max) ? fabs((*diff)(r,c)) : max;
		}
	}

	// calculate a scale factor
	float scaleFactor = 255.0f / max;
	
	// go through each node and draw the color
	QColor fillColor;
	for(int r = 0, h = hGridStep + hStart; r < diff->size(); r++, h += hGridStep)
	{
		for(int c = 0, w = wGridStep; c < diff->size(); c++, w += wGridStep)
		{
			int lightners = 255 - static_cast<int>(scaleFactor * abs((*diff)(r,c)));
			if((*diff)(r,c) >=0)
				fillColor.setRgb(lightners, 255, lightners);
			else
				fillColor.setRgb(255, lightners, lightners);

			brush.setColor(fillColor);

			int wid;
			int hei;

			if(c == diff->size() - 1)
				wid = width() - w;
			else	
				wid = wGridStep;

			if(r == diff->size() - 1)
				hei = height() - h;
			else
				hei = hGridStep;

			painter.fillRect(w, h, wid, hei, brush);
		}
	}


	// fill in the grid lines
	pen.setColor(QColor(100, 100, 100));
	
	// draw in the vertical grid lines
	for(int i = 0, w = wGridStep, h = hGridStep + hStart; 
		i < diff->size(); i++, 
		w += wGridStep, h += hGridStep)
	{
		if(i == 0)
			pen.setWidth(5);
		else
			pen.setWidth(1);

		painter.setPen(pen);

		painter.drawLine(w, hStart, w, height());
		painter.drawLine(0, h, width(), h);
	}

	// draw in the labels
	QRect boundBox;

	for(int i = 1; i <= diff->size(); i++)
	{
		// draw the row
		boundBox.setRect(i * wGridStep, hStart, wGridStep, hGridStep);
		pen.setColor(QColor(100, 100, 100));
		painter.setPen(pen);
		painter.drawText(boundBox, Qt::AlignHCenter | Qt::AlignVCenter, 
					diff->getConcept(i - 1));

		// draw the column
		boundBox.setRect(0, i * hGridStep + hStart, wGridStep, hGridStep);
		painter.drawText(boundBox, Qt::AlignHCenter | Qt::AlignVCenter, 
					diff->getConcept(i - 1));
		
	}
	

}
