/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */

#ifndef FRAME_H
#define FRAME_H

#include <QString>

#include <vector>
using namespace std;

class Frame
{
	public:
		Frame();
		Frame(const Frame & frame);
		const Frame & operator=(const Frame & frame);
		~Frame();

		// load a frame from a csv file
		// main diag is zeroed
		void load(const QString & file);

		// write a frame to a csv file
		// main diag is zeroed
		// current name is used as the file name
		void write(const QString & dir = "");

		// returns the concept weight between a and b,
		// will return 0.0 if a == b
		float & operator()(const QString & a, const QString & b);
		const float & operator()(const QString & a, const QString & b) const;

		// access as a = row b = column
		float & operator()(int a, int b);
		const float & operator()(int a, int b) const;

		// write the concept names as a vector
		void getConcepts(vector<QString> & names) const;
	
		// return the concept at a given index
		const QString & getConcept(int c) const;

		// return the size
		int size() const;

		// set the current size (no data is carried)
		void setSize(int size);

		// return the frame name (file name)
		const QString & name() const;

		// set the current frame name
		void setName(const QString & name);

		// return the raw data vector as a const pointer
		const float * raw() const;

		// clear the frame
		void clear();

	private:
		float * data;
		int frameSize;
		vector<QString> concepts;
		QString frameName;
};

#endif // FRAME_H

/**
 * a helper function that will return false if two 
 * frames are not compatible
 */
bool framesAreCompatible(const Frame & a, const Frame & b);

/**
 * a helper function that will check two frames and
 * throw and exception if the frames are not compatible
 */
void checkFrames(const Frame & a, const Frame & b);

/**
 * a helper function that will subtrack two frames
 */
void subtract(const Frame & a, const Frame & b, Frame & c);

