/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */

#include "distance.h"
#include "GeneralException.h"

#include <cmath>
#include <iostream>
using namespace std;

// for reference, I put definitions at the top
// followed by the registerDistance function and getter
// followed by the distance implementation
//
// To add a distance
//
// 	1. add a definition that matches the frameDistance typedef in distances.h
// 	2. add a line to registerDiffs "frameDistance["<your dist>"] = <your dist>
// 	3. implement <your dist> at the bottom

static map<QString, frameDistance> frameDists;

// declarations
float simpleDistance(const Frame & a, const Frame & b);
float popStdevDistance(const Frame & a, const Frame & b);
float stdevDistance(const Frame & a, const Frame & b);

// functions
void registerDistanceFunctions()
{
	// add functions here!!!
	frameDists["Simple Distance"] = simpleDistance;	
	frameDists["Population Stdev"] = popStdevDistance;
	frameDists["Sample Stdev"] = stdevDistance;
}

frameDistance getFrameDistanceFunction(const QString & name)
{
	if(frameDists.find(name) == frameDists.end())
		return 0;

	return frameDists[name];
}

void getRegisteredDistanceFunctions(vector<QString> & names)
{
	for(map<QString, frameDistance>::iterator iter = frameDists.begin();
		iter != frameDists.end(); iter++)
		names.push_back(iter->first);
}

// implementations
float simpleDistance(const Frame & a, const Frame & b)
{
	// a simple distance will just sum the abs differences between
	// each of the frame components
	checkFrames(a,b);

	// sum the differences of each value in a and b and ouptut 
	// diffs to c
	float sum = 0.0f;
	for(int i = 0, stop = a.size(); i < stop; i++)
		for(int j = 0; j < stop; j++)
			sum += abs(a(i,j) - b(i,j));

	return sum;
}

float popStdevDistance(const Frame & a, const Frame & b)
{
	// a simple distance will just sum the abs differences between
	// each of the frame components
	checkFrames(a,b);

	// sum the differences of each value in a and b and ouptut 
	// diffs to c
	float sum = 0.0f;
	vector<float> sums;
	for(int i = 0, stop = a.size(); i < stop; i++)
	{
		for(int j = 0; j < stop; j++)
		{
			float s = abs(a(i,j) - b(i,j));
			sum += s;
			sums.push_back(s);
		}
	}

	// calulate the average
	sum /= static_cast<float>(sums.size());

	// calculate the population standard deviation
	float stdev = 0.0f;
	for(unsigned int i = 0, stop = sums.size(); i < stop; i++)
		stdev += (sum - sums[i])*(sum - sums[i]);

	stdev /= static_cast<float>(sums.size());
	stdev = sqrt(stdev);

	return stdev;
}

float stdevDistance(const Frame & a, const Frame & b)
{
	// a simple distance will just sum the abs differences between
	// each of the frame components
	checkFrames(a,b);

	// sum the differences of each value in a and b and ouptut 
	// diffs to c
	float sum = 0.0f;
	vector<float> sums;
	for(int i = 0, stop = a.size(); i < stop; i++)
	{
		for(int j = 0; j < stop; j++)
		{
			float s = abs(a(i,j) - b(i,j));
			sum += s;
			sums.push_back(s);
		}
	}

	// calulate the average
	sum /= static_cast<float>(sums.size());

	// calculate the population standard deviation
	float stdev = 0.0f;
	for(unsigned int i = 0, stop = sums.size(); i < stop; i++)
		stdev += (sum - sums[i])*(sum - sums[i]);

	stdev /= static_cast<float>(sums.size() - 1);
	stdev = sqrt(stdev);

	return stdev;
}

