/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */

#include "compare_widget.h"
#include "diff_factory.h"
#include "diff_widget.h"

#include <QHBoxLayout>
#include <QTabWidget>

#include <iostream>
using namespace std;

CompareWidget::CompareWidget(QWidget * parent) : QWidget(parent), layout(0), tabs(0)
{
	setWindowTitle("Frame Compare");
}

CompareWidget::~CompareWidget()
{
	// I expect that everything will be deleted by qt
}

void CompareWidget::loadFrames(const vector<Frame*> & frames)
{
	for(unsigned int i = 0; i < frameData.size(); i++)
	{
		if(frames[i])
			delete frames[i];
	}

	// we want a deep copy of the frame data so if the 
	// main app loads different frames we don't crash
	for(unsigned int i = 0; i < frames.size(); i++)
	{
		if(frames[i])
		{
			Frame * newFrame = new Frame(*frames[i]);
			frameData.push_back(newFrame);
		}
	}

	// this is where we need to load up the widgets
	if(layout)
		delete layout;	 // should delete tabs
	
	// create a new layout and tabs widget
	layout = new QHBoxLayout;

	tabs = new QTabWidget;

	// this is where we look for widgets that accept the number of frames

	map<QString, DiffWidget*> diffWidgets;
	getDiffWidgets(frameData, diffWidgets);
	
	// for each, create a dummy widget and add it to tabs
	for(map<QString, DiffWidget*>::iterator iter = diffWidgets.begin();
			iter != diffWidgets.end(); iter++)
		tabs->addTab(iter->second, iter->first);

	layout->addWidget(tabs);
	
	setLayout(layout);
}
