/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */

#ifndef DISTANCEWIDGET_H
#define DISTANCEWIDGET_H

#include <QObject>
#include <QWidget>
#include <QString>
#include <QColor> // for QRgb
#include "frame.h"

#include <map>
#include <vector>
using namespace std;

class QMouseEvent;
class QPaintEvent;
class QResizeEvent;
class QImage;
class QPixmap;
class QPainter;

class DistanceWidget : public QWidget
{
	Q_OBJECT

	public:
		DistanceWidget(QWidget * parent = 0);
		~DistanceWidget();

		void loadFrames(const vector<Frame*> & frames);
		void setMainFrame(const Frame * frame);
		void setDistName(const QString & name);

		void clear();

		void shouldUpdate();

		void getSelectedFrames(vector<Frame*> & frames);

	public slots:

		void loadMinDistance();
		void loadMaxDistance();

	protected:
		virtual void mouseMoveEvent(QMouseEvent * event);
		virtual void mouseReleaseEvent(QMouseEvent * event);
		virtual void mouseDoubleClickEvent(QMouseEvent * event);
		virtual void paintEvent(QPaintEvent * event);
		virtual void resizeEvent(QResizeEvent * event);

	private:
		void generateDists();
		void prepareImageBuffers();

		// drawing functions
		void drawNode(const Frame * frame, int x, int y, QPainter * painter, bool isMainNode = false, bool isPick = false);

		Frame * mainFrame;	
		vector<Frame*> frameData;
		vector<float> dists;
		float minDist;
		float maxDist;

		QImage * pickImage;	// image good for pixel access
		QPixmap * viewImage;	// pixmap good for drawing to screen

		bool mustRedrawImage;

		QString distName;

		// these maps are used to store picking color information
		map<QRgb, const Frame*> colorPickFrame;
		map<const Frame*, QRgb> framePickColor;

		// drawing parameters
		QColor nodeColor;
		QColor nodeOutline;
		QColor mainNodeColor;
		QColor mainNodeOutline;

		int nodeDiameter;

		vector<int> hovered;
		vector<int> selected;
		void clearHovered();
		void clearSelected();
};

#endif // DISTANCEWIDGET_H
