/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */

#include "main_widget.h"
#include "distance_widget.h"
#include "distance.h"
#include "GeneralException.h"
#include "compare_widget.h"

#include <QSplitter>
#include <QListWidget>
#include <QString>
#include <QStringList>
#include <QListWidgetItem>
#include <QHBoxLayout>
#include <QLabel>
#include <QVBoxLayout>
#include <QPushButton>
#include <QMessageBox>
#include <QFileDialog>

#include <iostream>
using namespace std;

MainWidget::MainWidget(QWidget * parent) : QWidget(parent)
{
	// set the window title
	setWindowTitle("Vis Design Challenge 1");

	// mainLayout is the base layout
	mainLayout = new QHBoxLayout;
	
	// selectDisplaySplitter sits in the HBoxLayout	
	selectDisplaySplitter = new QSplitter;	

		// the selectWidget is used to hold the dist function list	
		QWidget * selectWidget = new QWidget();
		selectLayout = new QVBoxLayout;
	
			loadFrameButton = new QPushButton("Load Frames");
			selectLayout->addWidget(loadFrameButton);

			// set up the vertical splitter
			selectSplitter = new QSplitter(Qt::Vertical);

				// dist func widget for holding the select widgets
				QWidget * distFuncWidget = new QWidget();
				QVBoxLayout * distFuncLayout = new QVBoxLayout;
				
				distFuncLabel = new QLabel("Distance Functions");
				distFuncLayout->addWidget(distFuncLabel);
	
				distanceFunctionList = new QListWidget;
				distFuncLayout->addWidget(distanceFunctionList);

				distFuncWidget->setLayout(distFuncLayout);	
				selectSplitter->addWidget(distFuncWidget);

				// frame select widget for holding frame select widgets
				QWidget * frameSelWidget = new QWidget();
				QVBoxLayout * frameSelLayout = new QVBoxLayout;
	
				frameLabel = new QLabel("Frames");
				frameSelLayout->addWidget(frameLabel);

				frameSelectWidget = new QListWidget;
				frameSelLayout->addWidget(frameSelectWidget);

				frameSelWidget->setLayout(frameSelLayout);
				selectSplitter->addWidget(frameSelWidget);

			selectLayout->addWidget(selectSplitter);
		
		selectWidget->setLayout(selectLayout);

	// dummy widget and layout for distance section
	QWidget * visWidget = new QWidget;
	distVisLayout = new QVBoxLayout;
	
		// the distance widget visualizes the distances	
		distanceWidget = new DistanceWidget;
		distVisLayout->addWidget(distanceWidget);

		// the Widget for the button layout
		QWidget * visButtonWidget = new QWidget;
		visButtonsLayout = new QHBoxLayout;

			// the min button
			loadMinDistance = new QPushButton("Load Min Distance");
			visButtonsLayout->addWidget(loadMinDistance);
			
			// the max button
			loadMaxDistance = new QPushButton("Load Max Distance");
			visButtonsLayout->addWidget(loadMaxDistance);

			// a spacer
			visButtonsLayout->addStretch();

			// the compare button
			compare = new QPushButton("Compare");
			visButtonsLayout->addWidget(compare);

		visButtonWidget->setLayout(visButtonsLayout);
		distVisLayout->addWidget(visButtonWidget);

	visWidget->setLayout(distVisLayout);

	// ad the two to the selectDisplaySplitter	
	selectDisplaySplitter->addWidget(selectWidget);
	selectDisplaySplitter->addWidget(visWidget);
	selectDisplaySplitter->setStretchFactor(1, 100);

	// add the splitter to the layout
	mainLayout->addWidget(selectDisplaySplitter);
	setLayout(mainLayout);

	// load up the distance function names 
	vector<QString> functions;
	getRegisteredDistanceFunctions(functions);
	for(unsigned int i = 0; i < functions.size(); i++)
		distanceFunctionList->addItem(functions[i]);

	// connect the double click slignal of the list to our distance selection function
	QObject::connect(distanceFunctionList, SIGNAL(itemDoubleClicked(QListWidgetItem*)),
			this, SLOT(distanceSelection(QListWidgetItem*)));

	// connect the loadFrame click to loadFrames function
	QObject::connect(loadFrameButton, SIGNAL(clicked(bool)), 
			this, SLOT(loadFrames()));
	
	// connect the loadMinDistance click to loadMinDistance function
	QObject::connect(loadMinDistance, SIGNAL(clicked(bool)), 
			distanceWidget, SLOT(loadMinDistance()));
	
	// connect the loadMaxDistance click to loadMaxDistance function
	QObject::connect(loadMaxDistance, SIGNAL(clicked(bool)), 
			distanceWidget, SLOT(loadMaxDistance()));
	
	// connect the double click of the function to the switch distance function
	QObject::connect(distanceFunctionList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), 
			this, SLOT(switchDistanceFunction(QListWidgetItem*)));

	QObject::connect(compare, SIGNAL(clicked(bool)), 
			this, SLOT(launchCompare()));
}

MainWidget::~MainWidget()
{
	// assume qt will delete things appropriately
	for(unsigned int i = 0; i < frames.size(); i++)
		delete frames[i];
}
		
void MainWidget::loadFrames()
{
	QStringList files = QFileDialog::getOpenFileNames(this, 
					"Load Frame Data", 
					"./");

	distanceWidget->clear();
	frameSelectWidget->clear();
	
	for(unsigned int i = 0; i < frames.size(); i++)
		delete frames[i];

	frames.clear();
	nameToFrame.clear();

	// load each frame and display a warning if a frame is unloadable
	for(int i = 0; i < files.count(); i++)
	{
		Frame * frame = new Frame;
		try {
			frame->load(files[i]);
			frames.push_back(frame);
			nameToFrame[frame->name()] = frame;
		} catch (GeneralException & e) {
			delete frame;
			QMessageBox::warning(this, "Warning!!!", "Failed to load file: " + files[i]);	
		}
	}

	distanceWidget->loadFrames(frames);
	
	// load the frame names into the list
	for(map<QString,Frame*>::iterator iter = nameToFrame.begin();
			iter != nameToFrame.end(); iter++)
		frameSelectWidget->addItem(iter->first);
		
	update();
}
		
void MainWidget::switchDistanceFunction(QListWidgetItem * item)
{
	distanceWidget->setDistName(item->text());
	distanceWidget->shouldUpdate();	
}

void MainWidget::distanceSelection(QListWidgetItem * item)
{
	distanceWidget->setDistName(item->text());
}

void MainWidget::launchCompare()
{
	CompareWidget * compareWidget = new CompareWidget;

	vector<Frame*> selectedFrames;
	distanceWidget->getSelectedFrames(selectedFrames);

	compareWidget->loadFrames(selectedFrames);
	
	QRect currentGeo = compareWidget->geometry();
	compareWidget->setGeometry(100, 100, 700, 700);

	compareWidget->show();
}
