/**
 * This file is part of cs838-dc1
 *
 * cs838-dc1 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 *  cs838-dc1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with cs838-dc1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Copyright 2010 James Hill
 *   */

#ifndef MAIN_WIDGET_H
#define MAIN_WIDGET_H

#include <QWidget>

#include "frame.h"
#include <vector>
#include <map>
using namespace std;

class DistanceWidget;
class QSplitter;
class QListWidget;
class QString;
class QListWidgetItem;
class QHBoxLayout;
class QVBoxLayout;
class QLabel;
class QPushButton;

class MainWidget : public QWidget
{
	Q_OBJECT

	public:
		MainWidget(QWidget * parent = 0);
		~MainWidget();	

	private slots:
		void distanceSelection(QListWidgetItem * item);
		void loadFrames();
		void switchDistanceFunction(QListWidgetItem * item);
		void launchCompare();

	private:

		// we'll attempt to show some ordering here
		// the main layout manager	
		QHBoxLayout * mainLayout;
		
			// the splitter between the selection widgets and
			// the visualization widget
			QSplitter * selectDisplaySplitter;

				// the layout for the distance function
				// and frame select widgets
				QVBoxLayout * selectLayout;

					// this is the button that loads up the frames
					QPushButton * loadFrameButton;	
		
					// this is splits the dist function and 
					// frame selection widgets	
					QSplitter * selectSplitter;	

						// the dist function label and selection		
						QLabel * distFuncLabel;
						QListWidget * distanceFunctionList;

						// the frame label and select
						QLabel * frameLabel;
						QListWidget * frameSelectWidget;

				// the layout for the vis section
				QVBoxLayout * distVisLayout;
					
					// the distance vis widget
					DistanceWidget * distanceWidget;

					// the button layout
					QHBoxLayout * visButtonsLayout;
					
						// load up the min distance
						QPushButton * loadMinDistance;

						// load up the max distance
						QPushButton * loadMaxDistance;

						// compare selecte Frames
						QPushButton * compare;

		// data members
		vector<Frame *> frames;
		map<QString, Frame*> nameToFrame;
};

#endif // MAIN_WIDGET_H
